import { applyDecorators, collectDecorators } from './metadata';
import { ModuleBuilder } from './module-builder';
import { DecoratorType } from './types';
import { decoratorUtil, injectUtil } from './utils';
function handleOptions(options) {
    if (typeof options === 'string') {
        return {
            namespace: options
        };
    }
    return options;
}
export function Module(pOptions) {
    return (target) => {
        collectDecorators(target);
        const options = handleOptions(pOptions);
        const moduleWrapper = function (...args) {
            const instance = new target(...args);
            const moduleBuilder = new ModuleBuilder(options.namespace, {});
            applyDecorators(moduleBuilder, instance);
            instance.__moduleBuilder__ = moduleBuilder;
            injectUtil.injectAll(instance);
            return instance;
        };
        moduleWrapper.prototype = target.prototype;
        return moduleWrapper;
    };
}
export function State() {
    return (target, propertyName) => {
        decoratorUtil.setDecorator(target, propertyName, DecoratorType.STATE);
    };
}
export function Inject(typeOrName) {
    return (target, propertyName, index) => {
        injectUtil.registerInjection(target, propertyName, typeOrName, index);
    };
}
export function Getter() {
    return (target, propertyName, descriptor) => {
        decoratorUtil.setDecorator(target, propertyName, DecoratorType.GETTER);
    };
}
export function Mutation() {
    return (target, propertyName, descriptor) => {
        decoratorUtil.setDecorator(target, propertyName, DecoratorType.MUTATION);
    };
}
export function Action() {
    return (target, propertyName, descriptor) => {
        decoratorUtil.setDecorator(target, propertyName, DecoratorType.ACTION);
    };
}
//# sourceMappingURL=decorators.js.map