import { DecoratorType } from './types';
import { getDecorators } from './utils/decorator-util';
export function collectDecorators(ctor) {
    const decorators = getDecorators(ctor);
    const proto = ctor.prototype;
    proto.__states__ = [];
    proto.__getters__ = [];
    proto.__mutations__ = [];
    proto.__actions__ = [];
    decorators.forEach((decorator, key) => {
        switch (decorator) {
            case DecoratorType.STATE:
                proto.__states__.push(key);
                break;
            case DecoratorType.GETTER:
                proto.__getters__.push(key);
                break;
            case DecoratorType.MUTATION:
                proto.__mutations__.push(key);
                break;
            case DecoratorType.ACTION:
                proto.__actions__.push(key);
                break;
        }
    });
}
export function applyDecorators(moduleBuilder, instance) {
    const constructor = instance.constructor;
    const proto = constructor.prototype;
    proto.__states__.forEach(propertyName => {
        applyState(moduleBuilder, instance, propertyName);
    });
    proto.__getters__.forEach(propertyName => {
        applyGetter(moduleBuilder, instance, propertyName);
    });
    proto.__mutations__.forEach(propertyName => {
        applyMutation(moduleBuilder, instance, propertyName);
    });
    proto.__actions__.forEach(propertyName => {
        applyAction(moduleBuilder, instance, propertyName);
    });
}
function applyState(moduleBuilder, instance, propertyName) {
    const value = instance[propertyName];
    Object.defineProperty(instance, propertyName, {
        get() {
            return moduleBuilder.state()[propertyName];
        },
        set(val) {
            moduleBuilder.state()[propertyName] = val;
        }
    });
    instance[propertyName] = value;
}
function applyGetter(moduleBuilder, instance, propertyName) {
    let getterFunction = instance.__lookupGetter__(propertyName) || ((state) => ({}));
    getterFunction = getterFunction.bind(instance);
    const getter = state => {
        return getterFunction();
    };
    moduleBuilder.addGetter(propertyName, getter);
    Object.defineProperty(instance, propertyName, {
        get() {
            return moduleBuilder.read(propertyName);
        }
    });
}
function applyMutation(moduleBuilder, instance, propertyName) {
    let mutationFunction = instance[propertyName] || ((payload) => undefined);
    mutationFunction = mutationFunction.bind(instance);
    const mutation = (state, payload) => {
        mutationFunction(payload);
    };
    moduleBuilder.addMutation(propertyName, mutation);
    instance[propertyName] = (payload) => {
        moduleBuilder.commit(propertyName, payload);
    };
}
function applyAction(moduleBuilder, instance, propertyName) {
    let actionFunction = instance[propertyName] || ((payload) => Promise.resolve());
    actionFunction = actionFunction.bind(instance);
    const action = (context, payload) => {
        return actionFunction(payload);
    };
    moduleBuilder.addAction(propertyName, action);
    instance[propertyName] = (payload) => {
        return moduleBuilder.dispatch(propertyName, payload);
    };
}
//# sourceMappingURL=metadata.js.map