export class ModuleBuilder {
    constructor(namespace, state) {
        this._namespace = namespace;
        this._module = {
            actions: {},
            getters: {},
            mutations: {},
            namespaced: true,
            state
        };
    }
    setStoreBuilder(storeBuilder) {
        this._storeBuilder = storeBuilder;
    }
    addAction(name, action) {
        if (this._module.actions) {
            this._module.actions[name] = action;
        }
    }
    addMutation(name, mutation) {
        if (this._module.mutations) {
            this._module.mutations[name] = mutation;
        }
    }
    addGetter(name, getter) {
        if (this._module.getters) {
            this._module.getters[name] = getter;
        }
    }
    state() {
        return this._storeBuilder && this._storeBuilder.store
            ? this._storeBuilder.store.state[this._namespace]
            : this._module.state;
    }
    commit(name, payload) {
        if (this._storeBuilder && this._storeBuilder.store) {
            return this._storeBuilder.store.commit(this.namespace + '/' + name, payload);
        }
        else {
            throw new Error('Could not commit: no store created');
        }
    }
    read(name) {
        if (this._storeBuilder && this._storeBuilder.store) {
            return this._storeBuilder.store.getters[this.namespace + '/' + name];
        }
        else {
            throw new Error('Could not commit: no store created');
        }
    }
    dispatch(name, payload) {
        if (this._storeBuilder && this._storeBuilder.store) {
            return this._storeBuilder.store.dispatch(this.namespace + '/' + name, payload);
        }
        else {
            throw new Error('Could not dispatch: no store created');
        }
    }
    get module() {
        return this._module;
    }
    get namespace() {
        return this._namespace;
    }
}
//# sourceMappingURL=module-builder.js.map