import { Container } from 'typedi';
import { Store } from 'vuex';
export class StoreBuilder {
    constructor(options = {}) {
        this.initialize(options);
    }
    get container() {
        return this._container;
    }
    get store() {
        return this._store;
    }
    initialize(options) {
        this._container = options.container || Container.of(undefined);
        this._options = {
            actions: options.actions || {},
            getters: options.getters || {},
            modules: options.modules || {},
            mutations: options.mutations || {},
            plugins: options.plugins || [],
            state: options.state,
            strict: options.strict || true
        };
    }
    useContainer(container) {
        this._container = container;
    }
    setStrict(strict) {
        this._options.strict = strict;
    }
    merge(options) {
        this._options = {
            actions: {
                ...this._options.actions,
                ...(options.actions || {})
            },
            getters: {
                ...this._options.getters,
                ...(options.getters || {})
            },
            modules: {
                ...this._options.modules,
                ...(options.modules || {})
            },
            mutations: {
                ...this._options.mutations,
                ...(options.mutations || {})
            },
            plugins: [...this._options.plugins, ...(options.plugins || [])],
            state: {
                ...this._options.state,
                ...(options.state || {})
            },
            strict: options.strict || this._options.strict
        };
    }
    loadModules(modules) {
        modules.forEach(moduleClass => {
            this.loadModule(moduleClass);
        });
    }
    loadModule(moduleClass) {
        let instance = this._container.get(moduleClass);
        if (!instance) {
            instance = new moduleClass();
            this._container.set(moduleClass, instance);
        }
        const moduleBuilder = instance.__moduleBuilder__;
        if (this._options.modules) {
            moduleBuilder.setStoreBuilder(this);
            this._options.modules[moduleBuilder.namespace] = moduleBuilder.module;
        }
    }
    addModule(namespace, moduleOptions) {
        if (this._options.modules) {
            this._options.modules[namespace] = moduleOptions;
        }
    }
    create() {
        this._store = new Store(this._options);
        this._store.__storeBuilder__ = this;
        return this._store;
    }
}
const storeBuilderSingleton = new StoreBuilder({
    state: {},
    strict: true
});
const namedStoreBuilderMap = {};
export function getStoreBuilder(name) {
    // the default store builder
    if (!name) {
        return storeBuilderSingleton;
    }
    // a named store builder
    const builder = namedStoreBuilderMap[name] ||
        (namedStoreBuilderMap[name] = new StoreBuilder({
            state: {},
            strict: true
        }));
    return builder;
}
//# sourceMappingURL=store-builder.js.map